﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Telerik.Windows.Controls.Map;
using System.Windows.Shapes;

namespace ImageProviderTest
{
    public class WriteableBitmapSource : ImageMapSource
	{
		private int baseZoomLevel = 4;
        private Size baseMapSize = new Size(1024, 464);

		internal WriteableBitmapProvider Provider
		{
			get;
			set;
		}

        public override bool IsLevelSupported(int level)
        {
            return level <= 4;
        }

		protected override ImageSource GetImageSource(int zoomLevel)
		{
            WriteableBitmap image;

            if (this.IsLevelSupported(zoomLevel) && this.Provider.MapControl != null)
			{
                double scale = Math.Pow(2, (zoomLevel - this.baseZoomLevel));

                LocationRect rect = this.Provider.GeoBounds;
				rect.MapControl = this.Provider.MapControl;

				Size pixelSize = this.GetPixelSize(rect);
                if (pixelSize.Width > 2 && pixelSize.Height > 2)
				{
					image = new WriteableBitmap((int)pixelSize.Width, (int)pixelSize.Height);

					ScaleTransform transform = new ScaleTransform();
					transform.ScaleX = scale;
					transform.ScaleY = scale;

                    Ellipse ellipse = new Ellipse()
                    {
                        Width = baseMapSize.Width,
                        Height = baseMapSize.Height,
                        Fill = new SolidColorBrush(Colors.Brown)
                    };

                    image.Render(ellipse, transform);
                    image.Invalidate();
				}
				else
				{
					image = new WriteableBitmap(1, 1);
                }
			}
			else
			{
				image = new WriteableBitmap(1, 1);
            }

			return image;
		}

		private Size GetPixelSize(LocationRect rect)
		{
			Point startPoint = rect.Northwest.GetPoint(this.Provider.MapControl);
			Point endPoint = rect.Southeast.GetPoint(this.Provider.MapControl);

			Size screen = new Size(Math.Abs(endPoint.X - startPoint.X), Math.Abs(endPoint.Y - startPoint.Y));
			return screen;
		}
	}
}
